Class: NilClass

Inherits:
Object
  • Object
show all
Defined in:
lib/xqsr3/extensions/string/map_option_string.rb

Overview

Standard NilClass extension for safely mapping an absent option string.

Instance Method Summary collapse

Instance Method Details

#map_option_string(*args) ⇒ Object

Returns nil because a nil option string cannot match a declared option.



15
16
17
18
# File 'lib/xqsr3/extensions/string/map_option_string.rb', line 15

def map_option_string *args

  nil
end