Class: String
- Inherits:
-
Object
- Object
- String
- Includes:
- Xqsr3::CommandLineUtilities::MapOptionString, Xqsr3::StringUtilities::EndsWith, Xqsr3::StringUtilities::NilIfEmpty, Xqsr3::StringUtilities::NilIfWhitespace, Xqsr3::StringUtilities::QuoteIf, Xqsr3::StringUtilities::StartsWith, Xqsr3::StringUtilities::ToSymbol, Xqsr3::StringUtilities::Truncate
- Defined in:
- lib/xqsr3/doc_.rb,
lib/xqsr3/extensions/string/to_bool.rb,
lib/xqsr3/extensions/string/quote_if.rb,
lib/xqsr3/extensions/string/truncate.rb,
lib/xqsr3/extensions/string/ends_with.rb,
lib/xqsr3/extensions/string/to_symbol.rb,
lib/xqsr3/extensions/string/starts_with.rb,
lib/xqsr3/extensions/string/nil_if_empty.rb,
lib/xqsr3/extensions/string/map_option_string.rb,
lib/xqsr3/extensions/string/nil_if_whitespace.rb
Overview
Standard class, extended with methods:
- String#ends_with?
- String#map_option_string
- String#nil_if_empty
- String#nil_if_whitespace
- String#quote_if
- String#starts_with?
- String#to_bool
- String#to_symbol
- String#truncate
Instance Method Summary collapse
-
#to_bool(**options) ⇒ Object
Attempts to convert instance to a Boolean value, based on the given
options.
Methods included from Xqsr3::StringUtilities::NilIfWhitespace
#nil_if_whitespace, string_nil_if_whitespace
Methods included from Xqsr3::CommandLineUtilities::MapOptionString
included, #map_option_string, map_option_string_from_string
Methods included from Xqsr3::StringUtilities::NilIfEmpty
#nil_if_empty, string_nil_if_empty
Methods included from Xqsr3::StringUtilities::StartsWith
#starts_with?, string_starts_with?
Methods included from Xqsr3::StringUtilities::ToSymbol
Methods included from Xqsr3::StringUtilities::EndsWith
#ends_with?, string_ends_with?
Methods included from Xqsr3::StringUtilities::Truncate
Methods included from Xqsr3::StringUtilities::QuoteIf
Instance Method Details
#to_bool(**options) ⇒ Object
Attempts to convert instance to a Boolean value, based on the given
options
See Xqsr3::Conversion::BoolParser
11 12 13 14 |
# File 'lib/xqsr3/extensions/string/to_bool.rb', line 11 def to_bool ** return ::Xqsr3::Conversion::BoolParser.to_bool self, ** end |