Module: Xqsr3::Diagnostics::Exceptions::WithCause

Defined in:
lib/xqsr3/diagnostics/exceptions/with_cause.rb

Overview

This inclusion module adds to an exception class the means to chain a cause (aka inner-exception), which is then exposed with the cause attribute.

Examples:

Passing an exception cause as a parameter

class ConfigurationError < StandardError
  include Xqsr3::Diagnostics::Exceptions::WithCause
end

cause = ArgumentError.new 'port is not numeric'
error = ConfigurationError.new cause

error.cause # => cause
error.chained_message # => 'port is not numeric'

Constant Summary collapse

INSPECT_HIDDEN_FIELDS =

Array of hidden fields.

[ 'has_implicit_message', 'uses_cause_message' ]

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#causeObject (readonly)

The cause / inner-exception, if any, specified to the instance initialiser. This attribute shadows Exception#cause (the interpreter's raise-chain cause); for includers, #cause refers to this module's @cause.



147
148
149
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 147

def cause
  @cause
end

#optionsObject (readonly)

The options passed to the initialiser, with :cause removed, if present.



151
152
153
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 151

def options
  @options
end

Instance Method Details

#chained_backtraceObject

A combination of the backtrace(s) of all chained exception(s).



198
199
200
201
202
203
204
205
206
207
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 198

def chained_backtrace

  b = backtrace

  return b unless cause

  cb = cause.respond_to?(:chained_backtrace) ? cause.chained_backtrace : cause.backtrace

  (cb - b) + b
end

#chained_message(**options) ⇒ Object

Message obtained by concatenation of all chained exceptions' messages.

Signature

  • Parameters:

    • options (+Hash+) Options that control the behaviour of the method;
  • Options:

    • :separator (+String+) A string used to separate each chained exception message. Defaults to ": ";


162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 162

def chained_message **options

  return message unless cause
  return message if @uses_cause_message

  m = message
  c = cause
  cm = c.respond_to?(:chained_message) ? c.chained_message(**options) : c.message

  return m if (cm || '').empty?
  return cm if (m || '').empty?

  sep = options[:separator] || ': '

  "#{m}#{sep}#{cm}"
end

#chaineesObject

An array of exceptions in the chain, excluding self.



180
181
182
183
184
185
186
187
188
189
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 180

def chainees

  return [] unless cause

  r = [ cause ]

  r += cause.chainees if cause.respond_to? :chainees

  r
end

#exceptionsObject

An array of exceptions in the chain, including self.



192
193
194
195
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 192

def exceptions

  [ self ] + chainees
end

#initialize(*args, **options) ⇒ Object

Defines an initializer for an exception class that allows a cause (aka an inner exception) to be specified, either as the first or last argument or as a :cause option.

Signature

  • Parameters:

    • args 0+ arguments passed through to the +include+-ing class' initialiser;
    • options (+Hash+) Options that control the behaviour of the method;
  • Options:

    • :cause - The exception to be used as a cause, and ensures that that is not inferred from the arguments. May be nil to ensure that no cause is inferred;


90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
# File 'lib/xqsr3/diagnostics/exceptions/with_cause.rb', line 90

def initialize(*args, **options)

  @uses_cause_message = false

  cz = options[:cause]

  if cz

    options = options.reject { |k, v| k == :cause }

    @has_implicit_message = args.empty?

    super(*args)

    warn 'unexpected implicit message' if @has_implicit_message && self.message != self.class.to_s

    @cause = cz
  else

    cz_ix = args.index { |arg| ::Exception === arg }

    if cz_ix

      args = args.dup

      cz = args.delete_at cz_ix

      if args.empty?

        if !(cz.message || '').empty? && cz.class.to_s != cz.message

          @uses_cause_message = true

          args = [ cz.message ]
        end
      end
    else

      cz = $!
    end

    @has_implicit_message = args.empty?

    super(*args)

    warn 'unexpected implicit message' if @has_implicit_message && self.message != self.class.to_s

    @cause = cz
  end

  @options = options
end