Module: Xqsr3::StringUtilities::StartsWith
- Included in:
- String
- Defined in:
- lib/xqsr3/string_utilities/starts_with.rb
Overview
+include+-able module that provides ::string_starts_with? and #starts_with? methods
Class Method Summary collapse
-
.string_starts_with?(s, *args) ⇒ Boolean
Reports on whether a string
sstarts with a given prefix or set of prefixes (+args+).
Instance Method Summary collapse
-
#starts_with?(*args) ⇒ Boolean
Reports on whether the instance starts with a given prefix or set of prefixes (+args+).
Class Method Details
.string_starts_with?(s, *args) ⇒ Boolean
Reports on whether a string s starts with a given prefix or set of
prefixes (+args+)
Signature
-
Parameters:
s(+String+) The string to be evaluated;args0+ arguments against whichswill be evaluated;
120 121 122 123 |
# File 'lib/xqsr3/string_utilities/starts_with.rb', line 120 def self.string_starts_with? s, *args StartsWith_Helper_.string_starts_with_array_ s, args end |
Instance Method Details
#starts_with?(*args) ⇒ Boolean
Reports on whether the instance starts with a given prefix or set of prefixes (+args+)
Signature
-
Parameters:
args0+ arguments against which the instance will be evaluated;
132 133 134 135 |
# File 'lib/xqsr3/string_utilities/starts_with.rb', line 132 def starts_with? *args StartsWith_Helper_.string_starts_with_array_ self, args end |