Module: Xqsr3::StringUtilities::EndsWith
- Included in:
- String
- Defined in:
- lib/xqsr3/string_utilities/ends_with.rb
Defined Under Namespace
Modules: EndsWith_Helper_
Class Method Summary collapse
-
.string_ends_with?(s, *args) ⇒ Boolean
Reports on whether a string
sends with a given prefix or set of prefixes (args).
Instance Method Summary collapse
Class Method Details
.string_ends_with?(s, *args) ⇒ Boolean
Reports on whether a string s ends with a given prefix or set of prefixes (args)
Signature
-
Parameters:
-
*Required parameters*:
s- String
-
The string to be evaluated
121 122 123 124 |
# File 'lib/xqsr3/string_utilities/ends_with.rb', line 121 def self.string_ends_with? s, *args EndsWith_Helper_.string_ends_with_array_ s, args end |
Instance Method Details
#ends_with?(*args) ⇒ Boolean
126 127 128 129 |
# File 'lib/xqsr3/string_utilities/ends_with.rb', line 126 def ends_with? *args EndsWith_Helper_.string_ends_with_array_ self, args end |